/*****************************************************************************************************
		Example code showing the DMA transfer from and to internal memory using Link port 1

		Data from the file tx_data.dat is stored into internal memory. This data is then transferred in 
		quad words from internal memory space to link port output channel 1	and then receives the data 
		back in again through link port 1 and places it in internal memory.
******************************************************************************************************/
#define __NOUNDERSCORES__							// Depending on the compiler patch installed, the register names as defined in sysreg.h 
													// may now have two underscores prepended so that they do not restrict the namespace 
													// of user programs. If the old names are required as in this example(without the 
													// underscores prepended), 
													// define the preprocessor macro __NOUNDERSCORES__ before including sysreg.h.
													// Only required with compiler patch 6.1.7 or later

#include "sysreg.h"
#include "signal.h"
#include "stdio.h"

#define N 64										// Number of words to transfer

int dm data_tx[N] = {								// Data to be transmitted to external memory
#include "tx_data.dat"
};

int dm data_tx[N];									// Data to be transmitted over link port
int dm data_rx[N];									// Data to be received over link port

struct TCB {
	int *DI;										// index
	int DX;											// count and stride in x direction
	int DY;											// count and stride in y direction
	int DP;											// DMA control word
};

struct TCB TCB_temp;								// Temp structure for programming TCBs

__builtin_quad q;									// Temp quad for programming TCBs

void dma5_int();									// Prototype for interrupt service routine
void dma9_int();

void main(void)
{
	int i=0;
	
	interrupt(SIGDMA5, dma5_int);					// Assign isr to DMA channel 5 (Link port 1 transmit)
	interrupt(SIGDMA9, dma9_int);					// Assign isr to DMA channel 9 (Link port 1 receive)
													
	TCB_temp.DI = data_tx;							// index points to source buffer
	TCB_temp.DX = 4 | (N << 16);					// modify is 4 for quad-word transfers, count is N and must be shifted to upper half 
	TCB_temp.DY = 0;								// only a 1 dimension DMA
	TCB_temp.DP = 0x47000000;						// control word set for quad-word transfers to internal memory with interrupt enabled

	__builtin_sysreg_write(LCTL1, 0x000004DA);

	q = __builtin_compose_128((long long)TCB_temp.DI | (long long)TCB_temp.DX << 32, (long long)(TCB_temp.DY | (long long)TCB_temp.DP << 32));
	__builtin_sysreg_write4(DC5, q);				// program the TCBs

	while(1)										// Endless loop
	i=i-i;
}


/*********************************************************************************************************************************************/
/*	DMA5 Interrupt Service Routine ***********************************************************************************************************/
/*********************************************************************************************************************************************/
void dma5_int()										// ISR for DMA channel 5
{		
	interrupt(SIGDMA9, dma9_int);					// Assign isr to DMA channel 9
	
	TCB_temp.DI = data_rx;							// index points to buffer in internal memory for received data
	TCB_temp.DP = 0x47000000;						// control word set for quad-word transfers to internal memory with interrupt enabled

	q = __builtin_compose_128((long long)TCB_temp.DI | (long long)TCB_temp.DX << 32, (long long)(TCB_temp.DY | (long long)TCB_temp.DP << 32));
	__builtin_sysreg_write4(DC9, q);				// program the TCBs

		
return;
}

/*********************************************************************************************************************************************/
/*	DMA9 Interrupt Service Routine ***********************************************************************************************************/
/*********************************************************************************************************************************************/
void dma9_int()										// ISR for DMA channel 9
{		
	printf("\nLink port DMAs completed\n"); 
	return;
}

